import NormalButton from '@/components/normalButton'
import Popup from '@/components/popup'
import { View } from '@tarojs/components'
import classnames from 'classnames'
import styles from './index.module.scss'
import React, { FC, memo, useCallback, useEffect, useMemo, useRef, useState } from 'react'
import Search from '@/components/search'
import IconFont from '@/components/iconfont/iconfont'
import { EnumDeliveryNoticeTypeList } from '@/api/index'
import dayjs from 'dayjs'
import { getFilterData } from '@/common/util'
import Taro from '@tarojs/taro'

interface SearchFilter {
  type?: number
  timeKey?: string
}

export interface SearchField extends Omit<SearchFilter, 'timeKey'> {
  date_min?: string
  date_max?: string
}

interface DeliveryPropsType {
  onSearchTime?: () => void
  onConfirm?: (searchField: SearchField, timeKey?: Pick<SearchFilter, 'timeKey'>['timeKey']) => void
  onCancal?: () => void
  onReset?: () => void
}

interface ButtonPropsType {
  isActive: boolean
  onClick?: Function
  children?: React.ReactNode
  customClassName?: string
}

export const filterTimeOptions = {
  0: {
    name: '不限',
    date_min: undefined,
    date_max: undefined,
  },
  1: {
    name: '今天',
    date_min: `${dayjs(new Date()).add(0, 'day').format('YYYY-MM-DD')} 00:00:00`,
    date_max: `${dayjs(new Date()).add(1, 'day').format('YYYY-MM-DD')} 00:00:00`,
  },
  2: {
    name: '昨日',
    date_min: `${dayjs(new Date()).add(-1, 'day').format('YYYY-MM-DD')} 00:00:00`,
    date_max: `${dayjs(new Date()).add(0, 'day').format('YYYY-MM-DD')} 00:00:00`,
  },
  3: {
    name: '近7日',
    date_min: `${dayjs(new Date()).add(-7, 'day').format('YYYY-MM-DD')} 00:00:00`,
    date_max: `${dayjs(new Date()).add(0, 'day').format('YYYY-MM-DD')} 00:00:00`,
  },
  4: {
    name: '近30日',
    date_min: `${dayjs(new Date()).add(-30, 'day').format('YYYY-MM-DD')} 00:00:00`,
    date_max: `${dayjs(new Date()).add(0, 'day').format('YYYY-MM-DD')} 00:00:00`,
  },
  5: {
    name: '近90日',
    date_min: `${dayjs(new Date()).add(-90, 'day').format('YYYY-MM-DD')} 00:00:00`,
    date_max: `${dayjs(new Date()).add(0, 'day').format('YYYY-MM-DD')} 00:00:00`,
  },
} as const

const DeliveryFilter = memo<DeliveryPropsType>((props) => {
  const { onSearchTime, onConfirm, onCancal, onReset } = props

  const { fetchData, state } = EnumDeliveryNoticeTypeList()

  const [searchFilter, setSearchFilter] = useState<SearchFilter>({
    type: undefined,
    timeKey: '0',
  })
  const handleSelectedType = (type: string | number) => {
    console.log('type==>', type)
    if (typeof type === 'string' && type === 'default') {
      setSearchFilter((e) => ({ ...e, type: undefined }))
    } else {
      setSearchFilter((e) => ({ ...e, type: type as number }))
    }
  }


  const typeList = useMemo<{ id: number; name: string }[]>(() => {
    return state.data.list
  }, [state])

  useEffect(() => {
    if (!!!typeList?.length) {
      getEnumData()
      console.log('getEnumData')
    }
  }, [typeList])

  const getEnumData = async () => {
    await fetchData()
  }

  const [customTime, setCustomTime] = useState({
    date_min: undefined,
    date_max: undefined,
  })

  

  const handleSelectedTime = (timeOptionId) => {
    setSearchFilter((e) => ({ ...e, timeKey: timeOptionId }))
    // 不是设置了自定义时间
    if (timeOptionId !== '6') {
      setCustomTime({
        date_min: filterTimeOptions[timeOptionId]['date_min'],
        date_max: filterTimeOptions[timeOptionId]['date_max'],
      })
      return
    } 
    onSearchTime?.()
  }

  const handleReset = () => {
    setSearchFilter({ type: undefined, timeKey: '0' })
    onReset?.()
  }
  const handleConfirm = () => {
    const changedField = {
      type: searchFilter.type,
      date_min: customTime.date_min,
      date_max: customTime.date_max,
    }
    console.log('changedField==>', changedField)
    onConfirm?.(changedField, searchFilter.timeKey)
  }

  return (
    <>
      <View className={styles.filterItem}>
        <View className={styles['filterItem--title']}>发货类型</View>
        <View className={classnames(styles['filterItem--wrapper'], styles['filter--type'])}>
          <FilterButton isActive={searchFilter.type == undefined} onClick={() => handleSelectedType('default')}>
            不限
          </FilterButton>
          {!!typeList?.length &&
            typeList?.map(item => {
              return (
                <FilterButton isActive={searchFilter.type === item?.id} onClick={() => handleSelectedType(item?.id)}>
                  {item.name}
                </FilterButton>
              )
            })}
        </View>
      </View>

      <View className={styles.filterItem}>
        <View className={styles['filterItem--title']}>查询日期</View>
        <View className={classnames(styles['filterItem--wrapper'], styles['filter--time'])}>
          {Object.entries(filterTimeOptions).map(([key, value]) => {
            return (
              <FilterButton isActive={searchFilter.timeKey === key} onClick={() => handleSelectedTime(key)}>
                {value.name}
              </FilterButton>
            )
          })}
          <FilterButton isActive={searchFilter.timeKey === '6'} customClassName={styles.filterTimeButton} onClick={() => handleSelectedTime('6')}>
            自定义时间<IconFont name='icon-chakanquanbukehu' size={40} color={searchFilter.timeKey === '6' ? '#4581ff' : '#909090'}></IconFont>
          </FilterButton>
        </View>
      </View>
      <View className={styles.filterController}>
        <NormalButton plain type='primary' round customClassName={classnames(styles.btn, styles.rest_btn)} onClick={handleReset}>
          重置
        </NormalButton>
        <NormalButton type='primary' round customClassName={classnames(styles.btn, styles.verify_btn)} onClick={handleConfirm}>
          确认
        </NormalButton>
      </View>
    </>
  )
})

const FilterButton: FC<ButtonPropsType> = (props) => {
  const { onClick, children, isActive = false, customClassName } = props
  const handleClick = () => {
    onClick?.()
  }
  return (
    <View className={classnames(styles.filterButton, isActive && styles['filterButton--active'], customClassName)} onClick={handleClick}>
      <View className={classnames(styles['filterButton--text'])}>{children}</View>
    </View>
  )
}

export default DeliveryFilter
